(*****************************************************************************
*                                                                           *
*  Copyright 1996 bei Stefan Graf (CompuServe: 100736.2464)                 *
*                                                                           *
*  !!!!!!!!!!!!!!!!!!!!!! Nicht registrierte Version   !!!!!!!!!!!!!!!!!!!! *                                              *
*                                                                           *
*****************************************************************************)

unit Capihand;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, CapiBase;

const
  MaxAnzahlBuffer = 4;

type
  TConnectStatus = (ConnectNone,ConnectWait,ConnectD,ConnectB1,ConnectB3);

  TProtokolB2  = (X75,TRANSHDLC,BITTRANS,SNA_SDLC,BTX_X75,FAX,LAPD,V110TRANS,V110SDLC,V110X75);
  TProtokolB3  = (T70NL,ISO8208,T90,TRANSPARENT,T30);
  TCapiService = (TELEPHONY,AB_SERVICE,X21,FAX4,BTX,DATA_64K,
                  X25,TTX64,MIXED_MODE,TEMEX,REMUTE,GRAPHIC_TELEPHONY,
                  NEW_BTX,VIDEO_TELEPHONY,UNKNOWN);
  TServiceSet  = set of TCapiService;

  TCapiInfo   = (Gebuehren,DatumZeit,Display,UserToUser,DisconnectCause,Status);
  TCapiEAZ    = (Global,EAZ_1,EAZ_2,EAZ_3,EAZ_4,EAZ_5,EAZ_6,EAZ_7,EAZ_8,EAZ_9);
  TInfoSet    = set of TCapiInfo;
  TEAZSet     = set of TCapiEAZ;
  TCallNumber = String [20];
  TInfoDaten  = CapiParameterType;
  TVerbindung = (Active,Passive);

  TCapiDisconnectEvent   = procedure (Sender:TObject; const Grund : Word) of object;
  TCapiRingEvent         = function  (Sender           : TObject;
                                      const Service    : TCapiService;
                                      const ServiceAdd : Word;
                                      const CallFrom   : TCallNumber;
                                      const CallTo     : TCallNumber) : Word of object;

  TCapiConnectEvent      = procedure (Sender:TObject) of object;
  TCapiConnectB3Event    = function  (Sender:TObject) : Boolean of object;
  TCapiInfoEvent         = procedure (Sender:TObject; const Info : Word ; var InfoDaten : TInfoDaten) of object;
  TCapiDataEvent         = procedure (Sender:TObject; var UserDaten : CapiParameterType) of object;
  TCapiReceiveEvent      = procedure (Sender:TObject; const Count : Word) of object;
  TCapiSendEvent         = procedure (Sender:TObject) of object;
  TCapiErrorEvent        = procedure (Sender:TObject; const CommandID, SubCommand, ErrorNumber : Word) of object;

  TCapiHandler = class(TCapiBase)
  private
     { Private-Deklarationen }
     SendDatenBuffer   : array [1..MaxAnzahlBuffer] of DatenBufferType;
     RecvDatenBuffer   : array [1..MaxAnzahlBuffer] of DatenBufferType;

     BufferMemSize,
     BufferAnzahl : Word;

     SendPending,
     RecvBytes,
     SendBytes,
     RecvInIndex,
     RecvOutIndex,
     SendInIndex,
     SendOutIndex  : Word;

     BlockNr : Word;

     ConnectStatus : TConnectStatus;

     FB2Protokol : TProtokolB2;
     FB3Protokol : TProtokolB3;

     FBuffers,
     FBufferSize : Word;

     EAZ : Char;
     FEAZ : TCapiEAZ;

     FInfoSet : TInfoSet;
     InfoMask : LongInt;

     Verbindung : TVerbindung;

     FMessage         : TCapiMessageEvent;
     FDisconnect      : TCapiDisconnectEvent;
     FDisconnectB3    : TCapiDisconnectEvent;
     FRing            : TCapiRingEvent;
     FConnect         : TCapiConnectEvent;
     FConnectB3       : TCapiConnectB3Event;
     FReceive         : TCapiReceiveEvent;
     FSend            : TCapiSendEvent;
     FInfo            : TCapiInfoEvent;
     FData            : TCapiDataEvent;

     procedure SetB2Protokol       (const Protokol : TProtokolB2);
     procedure SetB3Protokol       (const Protokol : TProtokolB3);
     procedure SetAnzahlBuffers    (const Anzahl   : Word);
     procedure SetBufferSize       (const Size     : Word);
     procedure SetInfoMask         (const Mask     : TInfoSet);
     procedure SetEAZ              (const EAZ      : TCapiEAZ);

     procedure AllocBuffers;
     procedure ReleaseBuffers;

     procedure DoMessage (const CommandID, SubCommand, MsgID : Word;
                            var Daten : IndicationDatenType); override;

     procedure DoSendData;
     procedure DoReceiveData (var daten : DataB3IndType);
     function  GetB2ProtokolDaten (const Protokol : TProtokolB2) : ProtokolB2Type;
     function  GetB3ProtokolDaten (const Protokol : TProtokolB3) : ProtokolB3Type;
  protected
    { Protected-Deklarationen }
    procedure Loaded; override;
  public
    { Public-Deklarationen }
    PLCI,
    NCCI  : Word;

    constructor Create (AOwner:TComponent); override;
    destructor  Destroy; override;

    function Read  (var Daten; const Count : Word) : Integer;
    function Write (var Daten; const Count : Word) : Integer;
    function Flush : Boolean;

    function GetSendBytes : Word;
    function GetReceiveBytes : Word;
    function GetConnectStatus : TConnectStatus;

    function EnableListen   (const EAZ : TEAZSet; const Service : TServiceSet) : Boolean;
    function Call           (const Number : TCallNumber; const Service : TCapiService) : Boolean;
    function DisconnectLine (const Grund : Word) : Boolean;
  published
    { Published-Deklarationen }
    property B2Protokol  : TProtokolB2 read FB2Protokol  write SetB2Protokol     default X75;
    property B3Protokol  : TProtokolB3 read FB3Protokol  write SetB3Protokol     default TRANSPARENT;
    property Buffers     : Word        read FBuffers     write SetAnzahlBuffers  default 2;
    property BufferSize  : Word        read FBufferSize  write SetBufferSize     default 1024;
    property Infos       : TInfoSet    read FInfoSet     write SetInfoMask       default [];
    property OwnEAZ      : TCapiEAZ    read FEAZ         write SetEAZ            default EAZ_1;


    {Ereignis-Propertys}

    property OnDisconnect    : TCapiDisconnectEvent  read FDisconnect      write FDisconnect;
    property OnRing          : TCapiRingEvent        read FRing            write FRing;
    property OnConnect       : TCapiConnectEvent     read FConnect         write FConnect;
    property OnDataConnect   : TCapiConnectB3Event   read FConnectB3       write FConnectB3;
    property OnReceive       : TCapiReceiveEvent     read FReceive         write FReceive;
    property OnSend          : TCapiSendEvent        read FSend            write FSend;
    property OnInfo          : TCapiInfoEvent        read FInfo            write FInfo;
    property OnDataInfo      : TCapiDataEvent        read FData            write FData;
    property OnMessage       : TCapiMessageEvent     read FMessage         write FMessage;
  end;

procedure Register;

implementation

const
  EAZWert        : array [0..9]  of Word = ($01,$02,$04,$08,$10,$20,$40,$80,$100,$200);
  CallServices   : array [0..13] of Word = (1,2,3,4,5,7,8,9,10,12,13,14,15,16);
  ListenServices : array [0..13] of Word = ($02,$04,$08,$10,$20,$80,$100,$200,$400,$1000,$2000,$4000,$8000,$01);
  Services       : array [1..16] of TCapiService =(TELEPHONY,AB_SERVICE,X21,
                                                   FAX4,BTX,UNKNOWN,DATA_64K,X25,TTX64,MIXED_MODE,UNKNOWN,
                                                   TEMEX,REMUTE,GRAPHIC_TELEPHONY,
                                                   NEW_BTX,VIDEO_TELEPHONY);

constructor TCapiHandler.Create (AOwner : TComponent);
var i : Word;
begin
  inherited Create(AOwner);

  FB2Protokol  := X75;
  FB3Protokol  := TRANSPARENT;
  FBuffers     := 2;
  FBufferSize  := 1024;

  SetEAZ (EAZ_1);

  InfoMask := 0;
  SetInfoMask ([]);

  ConnectStatus := ConnectNone;

  for i:=1 to MaxAnzahlBuffer do begin
    SendDatenBuffer [i].Buffer := Nil;
    RecvDatenBuffer [i].Buffer := Nil;
  end;

  Loaded;
end;

destructor TCapiHandler.Destroy;
begin
  inherited Destroy;
end;


procedure TCapiHandler.Loaded;
begin
  inherited Loaded;
end;

procedure TCapiHandler.DoMessage (const CommandID, SubCommand, MsgID : Word;
                             var Daten : IndicationDatenType);
var
  flag,
  used     : Boolean;
  i,
  res    : Word;
  req    : RequestDatenType;
  resp   : RespondDatenType;

begin
  used := true;

  if (SubCommand = CONF) then begin
    case CommandID of
      CONNECT : begin
        if (daten.ConnectConf.Info <> 0) then
          SetError (CommandID,SubCommand,daten.InfoConf.Info)
        else begin
          ConnectStatus := ConnectWait;

          PLCI := daten.ConnectConf.PLCI;
        end;
      end;

      INFO : begin
        if (daten.InfoConf.Info <> 0) then
          SetError (CommandID,SubCommand,daten.InfoConf.Info);
      end;

      LISTEN : begin
        if (daten.ListenConf.Info <> 0) then
          SetError (CommandID,SubCommand,daten.ListenConf.Info);
      end;

      DATA_B3 : begin
        if (daten.DataB3Conf.Info <> 0) then
          SetError (CommandID,SubCommand,daten.DataB3Conf.Info);

        Dec (SendPending);
        Dec (SendBytes, SendDatenBuffer [SendOutIndex].ByteCount);

        SendDatenBuffer [SendOutIndex].ByteCount := 0;
        SendDatenBuffer [SendOutIndex].ReadyToSend := false;

        if Assigned (FSend) then
          FSend (Self);

        if (SendOutIndex >= BufferAnzahl) then
          SendOutIndex := 1
        else Inc (SendOutIndex);

        DoSendData;
      end;

      DISCONNECT : begin
        if (ConnectStatus <> ConnectNone) and Assigned (FDisconnect) then
          FDisconnect (Self,daten.DisconnectInd.Grund);

        ReleaseBuffers;

        ConnectStatus := ConnectNone;
      end;

      else
        used := false;
    end;
  end else begin
    case CommandID of
      CONNECT : begin
        PLCI := daten.ConnectInd.PLCI;

        if Assigned (FRing) then begin
          with daten.ConnectInd do
            res := FRing (Self, Services [ReqService], ReqServiceAdd, CallerAdr,Chr (ReqEAZ));
        end else res := 1;

        if (res = 0) then begin
          Verbindung := Passive;

          req.SelectB2ProtocolReq.PLCI      := daten.ConnectInd.PLCI;
          req.SelectB2ProtocolReq.Protokoll := Ord (FB2Protokol) + 1;
          req.SelectB2ProtocolReq.DLPD      := GetB2ProtokolDaten (FB2Protokol);
          SendRequestMessage (SELECT_B2_PROTOCOL,req);

          req.SelectB3ProtocolReq.PLCI      := daten.ConnectInd.PLCI;
          req.SelectB3ProtocolReq.Protokoll := Ord (FB3Protokol) + 1;
          req.SelectB3ProtocolReq.NCPD      := GetB3ProtokolDaten (FB3Protokol);
          SendRequestMessage (SELECT_B3_PROTOCOL,req);

          req.ListenB3Req.PLCI := daten.ConnectInd.PLCI;
          SendRequestMessage (LISTEN_B3,req);
        end;

        resp.ConnectResp.PLCI   := daten.ConnectInd.PLCI;
        resp.ConnectResp.Reject := res;
        SendResponsMessage (CONNECT,msgid,resp);
      end;


      CONNECT_ACTIVE : begin
        PLCI := daten.ActiveConnectInd.PLCI;
        ConnectStatus := ConnectD;

        resp.ActiveConnectResp.PLCI := PLCI;
        SendResponsMessage (CONNECT_ACTIVE,msgid,resp);

        if (Verbindung = Passive) then begin
          if Assigned (FConnect) then
            FConnect (Self);
        end else begin
          if Assigned (FConnectB3) then
            flag := FConnectB3 (Self)
          else flag := True;

          if (flag) then begin
            req.SelectB2ProtocolReq.PLCI      := PLCI;
            req.SelectB2ProtocolReq.Protokoll := Ord (FB2Protokol) + 1;
            req.SelectB2ProtocolReq.DLPD      := GetB2ProtokolDaten (FB2Protokol);
            SendRequestMessage (SELECT_B2_PROTOCOL,req);

            req.SelectB3ProtocolReq.PLCI      := PLCI;
            req.SelectB3ProtocolReq.Protokoll := Ord (FB3Protokol) + 1;
            req.SelectB3ProtocolReq.NCPD      := GetB3ProtokolDaten (FB3Protokol);
            SendRequestMessage (SELECT_B3_PROTOCOL,req);

            req.ConnectB3Req.PLCI := PLCI;
            req.ConnectB3Req.NCPI := '';
            SendRequestMessage (CONNECT_B3,req);
          end;
        end;
      end;


      CONNECT_B3 : begin
        if Assigned (FConnectB3) then
          flag := FConnectB3 (Self)
        else flag := True;

         resp.ConnectB3Resp.NCCI := daten.ConnectB3Ind.NCCI;
         resp.ConnectB3Resp.NCPI := #0;
         if (flag) then
           resp.ConnectB3Resp.Reject := 0
         else resp.ConnectB3Resp.Reject := 1;
         SendResponsMessage (CONNECT_B3,msgid,resp);
      end;

      CONNECT_B3_ACTIVE : begin
         NCCI := daten.ActiveB3ConnectInd.NCCI;

         AllocBuffers;

         ConnectStatus := ConnectB3;

         resp.ActiveB3ConnectResp.NCCI := NCCI;
         SendResponsMessage (CONNECT_B3_ACTIVE,msgid,resp);
      end;


      DISCONNECT_B3 : begin
        if (ConnectStatus = ConnectB3) and Assigned (FDisconnectB3) then
          FDisconnectB3 (Self,daten.DisconnectB3Ind.Grund);

        ConnectStatus := ConnectD;

        ReleaseBuffers;

        resp.DisconnectB3Resp.NCCI := daten.DisconnectB3Ind.NCCI;
        SendResponsMessage (DISCONNECT_B3,msgid,resp);

        if (ConnectStatus <> ConnectNone) then begin
          req.DisconnectReq.PLCI  := PLCI;
          req.DisconnectReq.Grund := 0;
          SendRequestMessage (DISCONNECT,req);
        end;
      end;


      DISCONNECT : begin
        if (ConnectStatus <> ConnectNone) and Assigned (FDisconnect) then
          FDisconnect (Self,daten.DisconnectInd.Grund);

        ConnectStatus := ConnectNone;

        resp.DisconnectResp.PLCI := daten.DisconnectInd.PLCI;
        SendResponsMessage (DISCONNECT,msgid,resp);
      end;


      DATA : begin
        if Assigned (FData) then
          FData (Self,daten.DataInd.UserDaten);

        resp.DataResp.PLCI := daten.DataInd.PLCI;
        SendResponsMessage (DATA,msgid,resp);
      end;


      DATA_B3 : begin
        DoReceiveData (daten.DataB3Ind);

        {Empfang und bernahme quittieren}
        resp.DataB3Resp.NCCI    := daten.DataB3Ind.NCCI;
        resp.DataB3Resp.BlockNr := daten.DataB3Ind.BlockNr;

        SendResponsMessage (DATA_B3, msgid, resp);
      end;

      INFO : begin
        if Assigned (FInfo) then
          FInfo (Self, daten.InfoInd.InfoNr, daten.InfoInd.Info);

        resp.InfoResp.PLCI := daten.InfoInd.PLCI;
        SendResponsMessage (INFO,msgid,resp);
      end;

      else
        used := false;
    end;
  end;

  if (not (used) and Assigned (FMessage)) then
    FMessage (Self,CommandID,SubCommand,msgid,daten);
end;


function TCapiHandler.GetB2ProtokolDaten (const Protokol : TProtokolB2) : ProtokolB2Type;
var dlpd : ProtokolB2Type;
begin
  FillChar (dlpd,SizeOf (dlpd),0);

  with dlpd do begin
    Size       := 6;
    DataLength := FBufferSize;
    LinkAdrA   := 3;
    LinkAdrB   := 1;
    ModuloMode := 8;
    WindowSize := 7;
    XID [0]    := #0;

    case FB2Protokol of
      SNA_SDLC : begin
        DataLength := 262;
        LinkAdrA   := $C1;
      end;

      FAX : begin
      end;

      LAPD : begin
      end;

      V110TRANS : begin
        V110_UserRate     := $40;
        V110_XID [0] := #0;
      end;

      V110SDLC : begin
        V110_UserRate     := $AF;
        V110_XID [0] := #0;
      end;

      V110X75 : begin
        V110_UserRate     := $40;
        V110_XID [0] := #0;
      end;
    end;
  end;

  GetB2ProtokolDaten := dlpd;
end;
                    

function TCapiHandler.GetB3ProtokolDaten (const Protokol : TProtokolB3) : ProtokolB3Type;
var ncpd : ProtokolB3Type;
begin
  FillChar (ncpd,SizeOf (ncpd),0);

  with ncpd do begin
    Size := 13;
    
    case Protokol of
      T70NL :;
      ISO8208 :;
      T90 :;
      TRANSPARENT :;
      T30 :;
    end;
  end;

  GetB3ProtokolDaten := ncpd;
end;

procedure TCapiHandler.AllocBuffers;
var i : Word;
begin
  BufferMemSize   := FBufferSize;
  BufferAnzahl := FBuffers;

  SendPending := 0;
  RecvBytes   := 0;
  SendBytes   := 0;

  RecvInIndex  := 1;
  RecvOutIndex := 1;
  SendInIndex  := 1;
  SendOutIndex := 1;

  for i:=1 To BufferAnzahl do begin
    GetMem (RecvDatenBuffer [i].Buffer,BufferMemSize);
    GetMem (SendDatenBuffer [i].Buffer,BufferMemSize);

    SendDatenBuffer [i].ByteIndex   := 0;
    SendDatenBuffer [i].ByteCount   := 0;
    SendDatenBuffer [i].ReadyToSend := False;
  end;
end;


procedure TCapiHandler.ReleaseBuffers;
var i: Word;
begin
  for i:=1 To MaxAnzahlBuffer do begin
    if (RecvDatenBuffer [i].Buffer <> Nil) then
      FreeMem (RecvDatenBuffer [i].Buffer,BufferSize);

    RecvDatenBuffer [i].Buffer := Nil;

    if (SendDatenBuffer [i].Buffer <> Nil) then
      FreeMem (SendDatenBuffer [i].Buffer,BufferSize);

    SendDatenBuffer [i].Buffer := Nil;
  end;
end;


procedure TCapiHandler.DoReceiveData (var daten : DataB3IndType);
var resp : RespondDatenType;
begin
  RecvDatenBuffer [RecvInIndex].ByteCount := daten.DataSize;
  RecvDatenBuffer [RecvInIndex].BlockNr   := daten.BlockNr;
  RecvDatenBuffer [RecvInIndex].ByteIndex := 1;

  Move (daten.DataPtr^,RecvDatenBuffer [RecvInIndex].Buffer^,daten.DataSize);

  Inc (RecvBytes, daten.DataSize);

  if (RecvInIndex >= BufferAnzahl) then
    RecvInIndex := 1
  else Inc (RecvInIndex);

  if Assigned (FReceive) then
    FReceive (Self, RecvBytes);
end;


procedure TCapiHandler.DoSendData;
var datareq : DataB3ReqType;
begin
  if (SendDatenBuffer [SendOutIndex].ReadyToSend) then begin
    datareq.NCCI    := NCCI;
    datareq.Size    := SendDatenBuffer [SendOutIndex].ByteCount;
    datareq.Daten   := SendDatenBuffer [SendOutIndex].Buffer;
    datareq.BlockNr := BlockNr;
    datareq.Flags   := 0;

    SendDatenBuffer [SendOutIndex].BlockNr := BlockNr;

    if (BlockNr >= BufferAnzahl) then
      BlockNr := 0
    else Inc (BlockNr);

    Inc (SendPending);

    SendRequestMessage (DATA_B3, datareq);
  end;
end;

procedure TCapiHandler.SetB2Protokol (const Protokol : TProtokolB2);
begin
  if (FB2Protokol <> Protokol) then begin
    FB2Protokol := Protokol;
  end;
end;

procedure TCapiHandler.SetB3Protokol (const Protokol : TProtokolB3);
begin
  if (FB3Protokol <> Protokol) then begin
    FB3Protokol := Protokol;
  end;
end;

procedure TCapiHandler.SetAnzahlBuffers (const Anzahl : Word);
begin
  if (FBuffers <> Anzahl) then begin
    FBuffers := Anzahl;
  end;
end;

procedure TCapiHandler.SetBufferSize (const Size : Word);
begin
  if (FBufferSize <> Size) then begin
    FBufferSize := Size;
  end;
end;

procedure TCapiHandler.SetInfoMask (const Mask : TInfoSet);
var daten : InfoReqType;
begin
  if (FInfoSet <> Mask) then begin
    FInfoSet := Mask;

    InfoMask := 0;

    if (Gebuehren in FInfoSet) then
      InfoMask := InfoMask Or InfoGebuehren;
    if (DatumZeit in FInfoSet) then
      InfoMask := InfoMask Or InfoDatumZeit;
    if (Display in FInfoSet) then
      InfoMask := InfoMask Or InfoDisplay;
    if (UserToUser in FInfoSet) then
      InfoMask := InfoMask Or InfoUserToUser;
    if (DisconnectCause in FInfoSet) then
      InfoMask := InfoMask Or InfoCause;
    if (Status in FInfoSet) then
      InfoMask := InfoMask Or InfoStatus;

    if (ConnectStatus <> ConnectNone) then begin
      daten.PLCI := PLCI;
      daten.InfoMask := InfoMask;
    end;
  end;
end;

procedure TCapiHandler.SetEAZ (const EAZ: TCapiEAZ);
begin
  if (FEAZ <> EAZ) then begin
    FEAZ := EAZ;
    Self.EAZ := Chr (Ord (FEAZ) + Ord ('0'));
  end;
end;

function TCapiHandler.Read (var Daten; const Count : Word) : Integer;
var readanz : Word;
begin
  readanz := 0;

  if (RecvDatenBuffer [RecvOutIndex].ByteCount > 0) then begin
    if (RecvDatenBuffer [RecvOutIndex].ByteCount > Count) then
      readanz := Count
    else readanz := RecvDatenBuffer [RecvOutIndex].ByteCount;

    Move (RecvDatenBuffer [RecvOutIndex].Buffer^[RecvDatenBuffer [RecvOutIndex].ByteIndex], Daten, readanz);

    Dec (RecvBytes,readanz);

    Dec (RecvDatenBuffer [RecvOutIndex].ByteCount, readanz);
    Inc (RecvDatenBuffer [RecvOutIndex].ByteIndex, readanz);;

    if (RecvDatenBuffer [RecvOutIndex].ByteCount = 0) then begin
      if (RecvOutIndex >= BufferAnzahl) then
        RecvOutIndex := 1
      else Inc (RecvOutIndex);
    end;
  end;

  Read := readanz;
end;

function TCapiHandler.Write (var Daten; const Count : Word) : Integer;
var anz : Word;
begin
  if (ConnectStatus <> ConnectB3) then
    Write := 0
  else begin
    if (Count > 0) then begin
      while (SendDatenBuffer [SendInIndex].ReadyToSend) do
        Application.ProcessMessages;

      SendDatenBuffer [SendInIndex].ByteCount := Count;
      SendDatenBuffer [SendInIndex].ByteIndex := 1;

      Move (Daten, SendDatenBuffer [SendInIndex].Buffer^, Count);

      SendDatenBuffer [SendInIndex].ReadyToSend := True;

      if (SendInIndex >= BufferAnzahl) then
        SendInIndex := 1
      else Inc (SendInIndex);

      Inc (SendBytes, Count);

      if (SendPending < 1) then
        DoSendData;
    end;

    Write := Count;
  end;
end;

function TCapiHandler.Flush : Boolean;
begin
  if (ConnectStatus = ConnectB3) then begin
    DoSendData;

    while (SendDatenBuffer [SendInIndex].ReadyToSend) do
      Application.ProcessMessages;
  end;

  Flush := True;
end;

function TCapiHandler.GetSendBytes : Word;
begin
  GetSendBytes := SendBytes;
end;

function TCapiHandler.GetReceiveBytes : Word;
begin
  GetReceiveBytes := RecvBytes;
end;

function TCapiHandler.GetConnectStatus : TConnectStatus;
begin
  GetConnectStatus := ConnectStatus;
end;

function TCapiHandler.EnableListen (const EAZ : TEAZSet; const Service : TServiceSet) : Boolean;
var
  i     : Word;
  daten : ListenReqType;
begin
  daten.Controller := 0;
  daten.InfoMask := InfoMask;

  daten.EAZMask := 0;
  for i:=0 to 9 do begin
    if (TCapiEAZ (i) in EAZ) then
      daten.EAZMask := daten.EAZMask Or EAZWert [i];
  end;

  daten.SIMask := 0;
  for i:=0 to 13 do begin
    if (TCapiService (i) in Service) then
      daten.SIMask := daten.SIMask Or ListenServices [i];
  end;

  SendRequestMessage (LISTEN,daten);

  EnableListen := True;
end;

function TCapiHandler.Call (const Number : TCallNumber; const Service : TCapiService) : Boolean;
var daten : ConnectReqType;
begin
  daten.Controller := 0;
  daten.BKanal := $83;
  daten.InfoMask := InfoMask;
  daten.Service := CallServices [Ord (Service)];
  daten.ServiceAdd := 0;
  daten.EAZ := EAZ;
  daten.Rufnummer := Chr ($80) + Number;

  SendRequestMessage (CONNECT,daten);

  Verbindung := Active;

  Call := True;
end;

function TCapiHandler.DisconnectLine (const Grund : Word) : Boolean;
var daten : DisconnectReqType;
begin
  if (ConnectStatus <> ConnectNone) then begin
    daten.PLCI := PLCI;
    daten.Grund := Grund;

    SendRequestMessage (DISCONNECT,daten);
  end;

  DisconnectLine := True;
end;

procedure Register;
begin
  RegisterComponents('Device', [TCapiHandler]);
end;

end.
